import { Files } from "../library/Files";
import { CppCreator } from "./CppCreator";

export class TypeRegistration
{
  static includesRegex = /\/\*INCLUDES\*\//;
  static classDBRegistrationsRegex = /\/\*CLASS-DB-REGISTRATIONS\*\//;

  static exceptions = 
  [
    "RJGodotHeaders.h"
  ];

  static async createRegistrations()
  {
    let registrationImplementation = await Files.loadUTF8( CppCreator.registerTypesPath );
    let allFiles = await Files.getFiles( CppCreator.outputPath );
    let filter = ( file:string ) =>
    {
      if ( TypeRegistration.exceptions.indexOf( file ) !== -1 )
      {
        return false;
      }
      
      return file.endsWith( ".h" ) && file != "register_types.h";
    };

    let headers = allFiles.filter( file => filter( file ) );

    let includeStatements = headers.map( h => `#include "./${h}"`);
    let includes = includeStatements.join( "\n" );  
    
    registrationImplementation = 
      registrationImplementation.replace( TypeRegistration.includesRegex, includes );

    let classNames = headers.map( h => h.replace( /\.h$/, "" ) );
    let registrators = classNames.map( cn => `ClassDB::register_class<${cn}>();`)

    let registrations = registrators.join( "\n  " );
    registrationImplementation = 
      registrationImplementation.replace( TypeRegistration.classDBRegistrationsRegex, registrations );

    await Files.saveUTF8( CppCreator.registerTypesPathOutput, registrationImplementation );

  } 
}