import { MemberInitializer } from "../MemberInitializer";
import { Field } from "../members/Field";
import { Member } from "../members/Member";
import { Method } from "../members/Method";
import { Signal } from "../members/Signal";

export abstract class BaseFactory
{
  _regex:RegExp;

  constructor( regex:RegExp )
  {
    this._regex = regex;
  }

  handles( nameDefinition:string )
  {
    return this._regex.test( nameDefinition );
  }

  abstract create( accessModifier:string, nameDefinition:string, memberData:any, members:Member[] ):void;

  createMember<T extends Member>( accessModifier:string, nameDefinition:string, body:any ) 
  {
    return BaseFactory.createMemberFrom( accessModifier, nameDefinition, body ) as T;
  }

  static createMemberFrom( accessModifier:string, nameDefinition:string, body:any )
  {
    
    let memberInitializer = new MemberInitializer();
    

    if ( nameDefinition.indexOf( "()" ) != -1)
    {
      memberInitializer.isMethod = true;
      memberInitializer.isVirtual = true;

      let typeRegex = /\(\)\s*(?:\:\s*(.+)\s*)?$/;
      let result = typeRegex.exec( nameDefinition );

      if ( result[ 1 ] )
      {
        memberInitializer.type = result[ 1 ];
      }

      nameDefinition = nameDefinition.replace( typeRegex, "" );
    }

    let names = nameDefinition.split( /\s+/ );
    memberInitializer.name = names[ names.length -1 ];

    for ( let i = 0; i < names.length - 1; i++ )
    {
      if ( "final" === names[ i ] )
      {
        memberInitializer.isVirtual = false;
      }

      if ( "static" === names[ i ] )
      {
        memberInitializer.isStatic = true;
      }

      if ( "signal" === names[ i ] )
      {
        memberInitializer.isSignal = true;
      }

    } 

    let member:Member = null;

    if ( memberInitializer.isMethod )
    {
      if ( memberInitializer.isSignal )
      {
        member = new Signal( memberInitializer );
      }
      else
      {
        member = new Method( memberInitializer );
      }
      
    }
    else
    {
      member = new Field( memberInitializer );
    }
    
    member.accessModifier = accessModifier;
    member.parseBody( body );

    return member;  
  }
}