import { Member } from "../members/Member";
import { MemberInitializer } from "../MemberInitializer";
import { Method } from "../members/Method";
import { Field } from "../members/Field";
import { Signal } from "../members/Signal";
import { ActionFactory } from "./ActionFactory";
import { PropertyFactory } from "./PropertyFactory";
import { BaseFactory } from "./BaseFactory";


export class MemberFactory
{
  static readonly actionRegex = /^\s*action\s+(\w+)\s*$/;
  static readonly propertyRegex = /^\s*property(?:\s*\@\s*(\w+))?\s+(\w+)\s*$/;

  static factories:BaseFactory[] = 
  [
    new ActionFactory(),
    new PropertyFactory()
  ]

  static grabMembers( membersData:any, accessModifier:string )
  {
    let members:Member[] = [];

    if ( ! membersData )
    {
      return members;
    }

    for ( let it in membersData )
    {
      if ( ! membersData.hasOwnProperty( it ) )
      {
        continue;
      }

      let nameDefinition = it;    
      let memberData = membersData[ it ];

      let factory = this.factories.find( f => f.handles( nameDefinition ) );

      if ( factory )
      {
        factory.create( accessModifier, nameDefinition, memberData, members );
      }
      else
      {
        members.push( BaseFactory.createMemberFrom(  accessModifier, nameDefinition, memberData ) );
      }

      /*
      if ( MemberFactory.actionRegex.test( nameDefinition ) )
      {
        let name = MemberFactory.actionRegex.exec( nameDefinition )[ 1 ];
        let upperCaseName = name[ 0 ].toUpperCase() + name.substring( 1 );
        let actionName = `on${upperCaseName}`;

        let signal = `signal ${name}()`;

        members.push( MemberFactory.create( accessModifier, signal, {} ) );
        members.push( MemberFactory.create( accessModifier, actionName, "Node<RJAction>" ) );
      }
      else if ( MemberFactory.propertyRegex.test( nameDefinition ) )
      {
        let result = MemberFactory.propertyRegex.exec( nameDefinition );

        let signalName = result[ 1 ];
        let fieldName  = result[ 2 ];

        signalName = signalName || `${fieldName}Changed`;

        let upperCaseName = signalName[ 0 ].toUpperCase() + signalName.substring( 1 );
        let actionName = `on${upperCaseName}`;

        let signal = `signal ${signalName}()`;

        if ( ! result[ 1 ] )
        {
          members.push( MemberFactory.create( accessModifier, signal, {} ) );
          members.push( MemberFactory.create( accessModifier, actionName, "Node<RJAction>" ) );
        }         

        let field = MemberFactory.create( accessModifier, fieldName, memberData ) as Field;

        field.isProperty = true;
        field.propertySignalName = signalName;
        field.propertyActionName = actionName;

        members.push( field );
      }
      else
      {
        let classMember = MemberFactory.create(  accessModifier, nameDefinition, memberData );
        members.push( classMember );
      }

      */

    }

    return members;
  }

  /*static create( accessModifier:string, nameDefinition:string, body:any )
  {
    
    let memberInitializer = new MemberInitializer();
    

    if ( nameDefinition.indexOf( "()" ) != -1)
    {
      memberInitializer.isMethod = true;
      memberInitializer.isVirtual = true;

      let typeRegex = /\(\)\s*(?:\:\s*(.+)\s*)?$/;
      let result = typeRegex.exec( nameDefinition );

      if ( result[ 1 ] )
      {
        memberInitializer.type = result[ 1 ];
      }

      nameDefinition = nameDefinition.replace( typeRegex, "" );
    }

    let names = nameDefinition.split( /\s+/ );
    memberInitializer.name = names[ names.length -1 ];

    for ( let i = 0; i < names.length - 1; i++ )
    {
      if ( "final" === names[ i ] )
      {
        memberInitializer.isVirtual = false;
      }

      if ( "static" === names[ i ] )
      {
        memberInitializer.isStatic = true;
      }

      if ( "signal" === names[ i ] )
      {
        memberInitializer.isSignal = true;
      }

    } 

    let member:Member = null;

    if ( memberInitializer.isMethod )
    {
      if ( memberInitializer.isSignal )
      {
        member = new Signal( memberInitializer );
      }
      else
      {
        member = new Method( memberInitializer );
      }
      
    }
    else
    {
      member = new Field( memberInitializer );
    }
    
    member.accessModifier = accessModifier;
    member.parseBody( body );

    return member;  
  }

  */
}