import { Field } from "../members/Field";
import { Member } from "../members/Member";
import { BaseFactory } from "./BaseFactory";

export class PropertyFactory extends BaseFactory
{
  static readonly regex = /^\s*property\s+(\w+)\s*$/;

  constructor(){ super( PropertyFactory.regex ); }

  create( accessModifier:string, nameDefinition:string, memberData:any, members:Member[] )
  {
    let result = PropertyFactory.regex.exec( nameDefinition );

    let fieldName  = result[ 1 ];

    let upperCaseName = fieldName[ 0 ].toUpperCase() + fieldName.substring( 1 )

    let signalName = `${fieldName}Changed`;

    let upperCaseSignalName = signalName[ 0 ].toUpperCase() + signalName.substring( 1 );
    let actionName = `on${upperCaseSignalName}`;

    let signalMethodDefinition = `signal ${signalName}()`;

           

    let field = this.createMember<Field>( accessModifier, fieldName, memberData );

    field.isProperty = true;
    field.propertySignalName = signalName;
    field.propertyActionName = actionName;

    
    members.push( field );
    
    
    members.push( this.createMember( accessModifier, signalMethodDefinition, {} ) );
    members.push( this.createMember( accessModifier, actionName, "Node<RJAction>" ) );
    

    let dispatchChanged = this.createMember( accessModifier, 
      
      `final dispatch${upperCaseName}Changed()`, [ {},
        [
          `emit_signal( SNAME( "${signalName}" ) );`,
          "",
          `if ( ${actionName} != NULL )`,
          "{",
          `  ${actionName}->trigger();`,
          "}"
        ]
      ]
    );

    
    members.push( dispatchChanged ); 

    

    let setSilent = this.createMember( accessModifier, 
      
      `final set${upperCaseName}Silent()`, [ { "p_value": memberData}, 
        [
        "value = p_value;"
        ] 
     ]
    );

    members.push( setSilent ); 
    
  }
}